@@ -20,19 +20,16 @@ import threading
 import weakref
 
 from pymongo import thread_util
+from pymongo.common import HAS_SSL
 from pymongo.errors import ConnectionFailure, ConfigurationError
 
-
-have_ssl = True
 try:
-    import ssl
+    from ssl import match_hostname
 except ImportError:
-    have_ssl = False
-
-
-NO_REQUEST    = None
-NO_SOCKET_YET = -1
+    from pymongo.ssl_match_hostname import match_hostname
 
+if HAS_SSL:
+    import ssl
 
 if sys.platform.startswith('java'):
     from select import cpython_compatible_select as select
@@ -40,6 +37,10 @@ else:
     from select import select
 
 
+NO_REQUEST = None
+NO_SOCKET_YET = -1
+
+
 def _closed(sock):
     """Return True if we know socket has been closed, False otherwise.
     """
@@ -54,11 +55,13 @@ def _closed(sock):
 class SocketInfo(object):
     """Store a socket with some metadata
     """
-    def __init__(self, sock, pool_id):
+    def __init__(self, sock, pool_id, host=None):
         self.sock = sock
+        self.host = host
         self.authset = set()
         self.closed = False
         self.last_checkout = time.time()
+        self.forced = False
 
         # The pool's pool_id changes with each reset() so we can close sockets
         # created before the last reset.
@@ -95,22 +98,53 @@ class SocketInfo(object):
 # http://bugs.jython.org/issue1057
 class Pool:
     def __init__(self, pair, max_size, net_timeout, conn_timeout, use_ssl,
-                 use_greenlets):
+                 use_greenlets, ssl_keyfile=None, ssl_certfile=None,
+                 ssl_cert_reqs=None, ssl_ca_certs=None,
+                 wait_queue_timeout=None, wait_queue_multiple=None):
         """
         :Parameters:
           - `pair`: a (hostname, port) tuple
-          - `max_size`: approximate number of idle connections to keep open
+          - `max_size`: The maximum number of open sockets. Calls to
+            `get_socket` will block if this is set, this pool has opened
+            `max_size` sockets, and there are none idle. Set to `None` to
+             disable.
           - `net_timeout`: timeout in seconds for operations on open connection
           - `conn_timeout`: timeout in seconds for establishing connection
           - `use_ssl`: bool, if True use an encrypted connection
           - `use_greenlets`: bool, if True then start_request() assigns a
               socket to the current greenlet - otherwise it is assigned to the
               current thread
+          - `ssl_keyfile`: The private keyfile used to identify the local
+            connection against mongod.  If included with the ``certfile` then
+            only the ``ssl_certfile`` is needed.  Implies ``ssl=True``.
+          - `ssl_certfile`: The certificate file used to identify the local
+            connection against mongod. Implies ``ssl=True``.
+          - `ssl_cert_reqs`: Specifies whether a certificate is required from
+            the other side of the connection, and whether it will be validated
+            if provided. It must be one of the three values ``ssl.CERT_NONE``
+            (certificates ignored), ``ssl.CERT_OPTIONAL``
+            (not required, but validated if provided), or ``ssl.CERT_REQUIRED``
+            (required and validated). If the value of this parameter is not
+            ``ssl.CERT_NONE``, then the ``ssl_ca_certs`` parameter must point
+            to a file of CA certificates. Implies ``ssl=True``.
+          - `ssl_ca_certs`: The ca_certs file contains a set of concatenated
+            "certification authority" certificates, which are used to validate
+            certificates passed from the other end of the connection.
+            Implies ``ssl=True``.
+          - `wait_queue_timeout`: (integer) How long (in seconds) a
+            thread will wait for a socket from the pool if the pool has no
+            free sockets.
+          - `wait_queue_multiple`: (integer) Multiplied by max_pool_size to give
+            the number of threads allowed to wait for a socket at one time.
         """
-        if use_greenlets and not thread_util.have_greenlet:
+        # Only check a socket's health with _closed() every once in a while.
+        # Can override for testing: 0 to always check, None to never check.
+        self._check_interval_seconds = 1
+
+        if use_greenlets and not thread_util.have_gevent:
             raise ConfigurationError(
-                "The greenlet module is not available. "
-                "Install the greenlet package from PyPI."
+                "The Gevent module is not available. "
+                "Install the gevent package from PyPI."
             )
 
         self.sockets = set()
@@ -124,7 +158,17 @@ class Pool:
         self.max_size = max_size
         self.net_timeout = net_timeout
         self.conn_timeout = conn_timeout
+        self.wait_queue_timeout = wait_queue_timeout
+        self.wait_queue_multiple = wait_queue_multiple
         self.use_ssl = use_ssl
+        self.ssl_keyfile = ssl_keyfile
+        self.ssl_certfile = ssl_certfile
+        self.ssl_cert_reqs = ssl_cert_reqs
+        self.ssl_ca_certs = ssl_ca_certs
+
+        if HAS_SSL and use_ssl and not ssl_cert_reqs:
+            self.ssl_cert_reqs = ssl.CERT_NONE
+
         self._ident = thread_util.create_ident(use_greenlets)
 
         # Map self._ident.get() -> request socket
@@ -133,6 +177,14 @@ class Pool:
         # Count the number of calls to start_request() per thread or greenlet
         self._request_counter = thread_util.Counter(use_greenlets)
 
+        if self.wait_queue_multiple is None or self.max_size is None:
+            max_waiters = None
+        else:
+            max_waiters = self.max_size * self.wait_queue_multiple
+
+        self._socket_semaphore = thread_util.create_semaphore(
+            self.max_size, max_waiters, use_greenlets)
+
     def reset(self):
         # Ignore this race condition -- if many threads are resetting at once,
         # the pool_id will definitely change, which is all we care about.
@@ -149,7 +201,8 @@ class Pool:
         finally:
             self.lock.release()
 
-        for sock_info in sockets: sock_info.close()
+        for sock_info in sockets:
+            sock_info.close()
 
     def create_connection(self, pair):
         """Connect to *pair* and return the socket object.
@@ -199,7 +252,7 @@ class Pool:
             raise err
         else:
             # This likely means we tried to connect to an IPv6 only
-            # host with an OS/kernel or Python interpeter that doesn't
+            # host with an OS/kernel or Python interpreter that doesn't
             # support IPv6. The test case is Jython2.5.1 which doesn't
             # support IPv6 at all.
             raise socket.error('getaddrinfo failed')
@@ -210,19 +263,27 @@ class Pool:
            return_socket() when you're done with it.
         """
         sock = self.create_connection(pair)
+        hostname = (pair or self.pair)[0]
 
         if self.use_ssl:
             try:
-                sock = ssl.wrap_socket(sock)
+                sock = ssl.wrap_socket(sock,
+                                       certfile=self.ssl_certfile,
+                                       keyfile=self.ssl_keyfile,
+                                       ca_certs=self.ssl_ca_certs,
+                                       cert_reqs=self.ssl_cert_reqs)
+                if self.ssl_cert_reqs:
+                    match_hostname(sock.getpeercert(), hostname)
+
             except ssl.SSLError:
                 sock.close()
                 raise ConnectionFailure("SSL handshake failed. MongoDB may "
                                         "not be configured with SSL support.")
 
         sock.settimeout(self.net_timeout)
-        return SocketInfo(sock, self.pool_id)
+        return SocketInfo(sock, self.pool_id, hostname)
 
-    def get_socket(self, pair=None):
+    def get_socket(self, pair=None, force=False):
         """Get a socket from the pool.
 
         Returns a :class:`SocketInfo` object wrapping a connected
@@ -231,9 +292,11 @@ class Pool:
 
         :Parameters:
           - `pair`: optional (hostname, port) tuple
+          - `force`: optional boolean, forces a connection to be returned
+              without blocking, even if `max_size` has been reached.
         """
         # We use the pid here to avoid issues with fork / multiprocessing.
-        # See test.test_connection:TestConnection.test_fork for an example of
+        # See test.test_client:TestClient.test_fork for an example of
         # what could go wrong otherwise
         if self.pid != os.getpid():
             self.reset()
@@ -249,7 +312,18 @@ class Pool:
             checked_sock.last_checkout = time.time()
             return checked_sock
 
+        forced = False
         # We're not in a request, just get any free socket or create one
+        if force:
+            # If we're doing an internal operation, attempt to play nicely with
+            # max_size, but if there is no open "slot" force the connection
+            # and mark it as forced so we don't release the semaphore without
+            # having acquired it for this socket.
+            if not self._socket_semaphore.acquire(False):
+                forced = True
+        elif not self._socket_semaphore.acquire(True, self.wait_queue_timeout):
+            self._raise_wait_queue_timeout()
+
         sock_info, from_pool = None, None
         try:
             try:
@@ -265,6 +339,8 @@ class Pool:
         if from_pool:
             sock_info = self._check(sock_info, pair)
 
+        sock_info.forced = forced
+
         if req_state == NO_SOCKET_YET:
             # start_request has been called but we haven't assigned a socket to
             # the request yet. Let's use this socket for this request until
@@ -286,8 +362,6 @@ class Pool:
         return bool(self._request_counter.get())
 
     def end_request(self):
-        tid = self._ident.get()
-
         # Check if start_request has ever been called in this thread / greenlet
         count = self._request_counter.get()
         if count:
@@ -313,10 +387,19 @@ class Pool:
     def maybe_return_socket(self, sock_info):
         """Return the socket to the pool unless it's the request socket.
         """
+        # These sentinel values should only be used internally.
+        assert sock_info not in (NO_REQUEST, NO_SOCKET_YET)
+
         if self.pid != os.getpid():
+            if not sock_info.forced:
+                self._socket_semaphore.release()
             self.reset()
-        elif sock_info not in (NO_REQUEST, NO_SOCKET_YET):
+        else:
             if sock_info.closed:
+                if sock_info.forced:
+                    sock_info.forced = False
+                elif sock_info != self._get_request_state():
+                    self._socket_semaphore.release()
                 return
 
             if sock_info != self._get_request_state():
@@ -327,13 +410,19 @@ class Pool:
         """
         try:
             self.lock.acquire()
-            if len(self.sockets) < self.max_size:
+            if (len(self.sockets) < self.max_size
+                    and sock_info.pool_id == self.pool_id):
                 self.sockets.add(sock_info)
             else:
                 sock_info.close()
         finally:
             self.lock.release()
 
+        if sock_info.forced:
+            sock_info.forced = False
+        else:
+            self._socket_semaphore.release()
+
     def _check(self, sock_info, pair):
         """This side-effecty function checks if this pool has been reset since
         the last time this socket was used, or if the socket has been closed by
@@ -349,6 +438,9 @@ class Pool:
         """
         error = False
 
+        # How long since socket was last checked out.
+        age = time.time() - sock_info.last_checkout
+
         if sock_info.closed:
             error = True
 
@@ -356,7 +448,10 @@ class Pool:
             sock_info.close()
             error = True
 
-        elif time.time() - sock_info.last_checkout > 1:
+        elif (self._check_interval_seconds is not None
+                and (
+                    0 == self._check_interval_seconds
+                    or age > self._check_interval_seconds)):
             if _closed(sock_info.sock):
                 sock_info.close()
                 error = True
@@ -371,18 +466,19 @@ class Pool:
                 raise
 
     def _set_request_state(self, sock_info):
-        tid = self._ident.get()
+        ident = self._ident
+        tid = ident.get()
 
         if sock_info == NO_REQUEST:
             # Ending a request
-            self._ident.unwatch()
+            ident.unwatch(tid)
             self._tid_to_sock.pop(tid, None)
         else:
             self._tid_to_sock[tid] = sock_info
 
-            if not self._ident.watching():
-                # Closure over tid and poolref. Don't refer directly to self,
-                # otherwise there's a cycle.
+            if not ident.watching():
+                # Closure over tid, poolref, and ident. Don't refer directly to
+                # self, otherwise there's a cycle.
 
                 # Do not access threadlocals in this function, or any
                 # function it calls! In the case of the Pool subclass and
@@ -392,8 +488,10 @@ class Pool:
                 # thread locals in this function, while PyThreadState_Clear()
                 # is in progress can cause leaks, see PYTHON-353.
                 poolref = weakref.ref(self)
+
                 def on_thread_died(ref):
                     try:
+                        ident.unwatch(tid)
                         pool = poolref()
                         if pool:
                             # End the request
@@ -406,12 +504,18 @@ class Pool:
                         # Random exceptions on interpreter shutdown.
                         pass
 
-                self._ident.watch(on_thread_died)
+                ident.watch(on_thread_died)
 
     def _get_request_state(self):
         tid = self._ident.get()
         return self._tid_to_sock.get(tid, NO_REQUEST)
 
+    def _raise_wait_queue_timeout(self):
+        raise ConnectionFailure(
+            'Timed out waiting for socket from pool with max_size %r and'
+            ' wait_queue_timeout %r' % (
+                self.max_size, self.wait_queue_timeout))
+
     def __del__(self):
         # Avoid ResourceWarnings in Python 3
         for sock_info in self.sockets:
@@ -424,8 +528,8 @@ class Pool:
 
 class Request(object):
     """
-    A context manager returned by Connection.start_request(), so you can do
-    `with connection.start_request(): do_something()` in Python 2.5+.
+    A context manager returned by :meth:`start_request`, so you can do
+    `with client.start_request(): do_something()` in Python 2.5+.
     """
     def __init__(self, connection):
         self.connection = connection
