@@ -31,47 +31,41 @@ from pymongo.errors import ConnectionFailure, InvalidName
 from pymongo.errors import CollectionInvalid, OperationFailure
 from pymongo.errors import AutoReconnect
 from pymongo.database import Database
-from pymongo.connection import Connection
+from pymongo.mongo_client import MongoClient
 from pymongo.collection import Collection
 from pymongo.master_slave_connection import MasterSlaveConnection
+from test import host, port, host2, port2, host3, port3
 from test.utils import TestRequestMixin
-from test.test_connection import host, port
 
 class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
 
     def setUp(self):
-        # For TestRequestMixin:
-        self.host = host
-        self.port = port
-
-        self.master = Connection(host, port)
+        self.master = MongoClient(host, port)
 
         self.slaves = []
         try:
-            self.slaves.append(Connection(os.environ.get("DB_IP2", host),
-                               int(os.environ.get("DB_PORT2", 27018)),
-                               read_preference=ReadPreference.SECONDARY))
+            self.slaves.append(MongoClient(
+                host2, port2, read_preference=ReadPreference.SECONDARY))
         except ConnectionFailure:
             pass
 
         try:
-            self.slaves.append(Connection(os.environ.get("DB_IP3", host),
-                               int(os.environ.get("DB_PORT3", 27019)),
-                               read_preference=ReadPreference.SECONDARY))
+            self.slaves.append(MongoClient(
+                host3, port3, read_preference=ReadPreference.SECONDARY))
         except ConnectionFailure:
             pass
 
         if not self.slaves:
             raise SkipTest("Not connected to master-slave set")
 
-        self.connection = MasterSlaveConnection(self.master, self.slaves)
-        self.db = self.connection.pymongo_test
+        self.client = MasterSlaveConnection(self.master, self.slaves)
+        self.db = self.client.pymongo_test
 
     def tearDown(self):
         try:
             self.db.test.drop_indexes()
         except Exception:
-            # Tests like test_disconnect can monkey with the connection in ways
+            # Tests like test_disconnect can monkey with the client in ways
             # that make this fail
             pass
 
@@ -83,41 +77,41 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         self.assertRaises(TypeError, MasterSlaveConnection, self.master, [1])
 
     def test_use_greenlets(self):
-        self.assertFalse(self.connection.use_greenlets)
+        self.assertFalse(self.client.use_greenlets)
 
-        if thread_util.have_greenlet:
-            master = Connection(self.host, self.port, use_greenlets=True)
+        if thread_util.have_gevent:
+            master = MongoClient(host, port, use_greenlets=True)
             slaves = [
-                Connection(slave.host, slave.port, use_greenlets=True)
+                MongoClient(slave.host, slave.port, use_greenlets=True)
                 for slave in self.slaves]
 
             self.assertTrue(
                 MasterSlaveConnection(master, slaves).use_greenlets)
 
     def test_repr(self):
-        self.assertEqual(repr(self.connection),
+        self.assertEqual(repr(self.client),
                          "MasterSlaveConnection(%r, %r)" %
                          (self.master, self.slaves))
 
     def test_disconnect(self):
-        class Connection(object):
+        class MongoClient(object):
             def __init__(self):
                 self._disconnects = 0
 
             def disconnect(self):
                 self._disconnects += 1
 
-        self.connection._MasterSlaveConnection__master = Connection()
-        self.connection._MasterSlaveConnection__slaves = [Connection(),
-                                                          Connection()]
+        self.client._MasterSlaveConnection__master = MongoClient()
+        self.client._MasterSlaveConnection__slaves = [MongoClient(),
+                                                      MongoClient()]
 
-        self.connection.disconnect()
+        self.client.disconnect()
         self.assertEqual(1,
-            self.connection._MasterSlaveConnection__master._disconnects)
+            self.client._MasterSlaveConnection__master._disconnects)
         self.assertEqual(1,
-            self.connection._MasterSlaveConnection__slaves[0]._disconnects)
+            self.client._MasterSlaveConnection__slaves[0]._disconnects)
         self.assertEqual(1,
-            self.connection._MasterSlaveConnection__slaves[1]._disconnects)
+            self.client._MasterSlaveConnection__slaves[1]._disconnects)
 
     def test_continue_until_slave_works(self):
         class Slave(object):
@@ -130,7 +124,7 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
                 Slave.calls += 1
                 if self._fail:
                     raise AutoReconnect()
-                return 'sent'
+                return (None, 'sent')
 
         class NotRandomList(object):
             last_idx = -1
@@ -146,9 +140,9 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
                 NotRandomList.last_idx = idx
                 return self._items.pop(0)
 
-        self.connection._MasterSlaveConnection__slaves = NotRandomList()
+        self.client._MasterSlaveConnection__slaves = NotRandomList()
 
-        response = self.connection._send_message_with_response('message')
+        response = self.client._send_message_with_response('message')
         self.assertEqual((NotRandomList.last_idx, 'sent'), response)
         self.assertNotEqual(-1, NotRandomList.last_idx)
         self.assertEqual(3, Slave.calls)
@@ -177,10 +171,10 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
             def __getitem__(self, idx):
                 return self._items.pop(0)
 
-        self.connection._MasterSlaveConnection__slaves = NotRandomList()
+        self.client._MasterSlaveConnection__slaves = NotRandomList()
 
         self.assertRaises(AutoReconnect,
-            self.connection._send_message_with_response, 'message')
+            self.client._send_message_with_response, 'message')
         self.assertEqual(4, Slave.calls)
 
     def test_get_db(self):
@@ -188,57 +182,57 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         def make_db(base, name):
             return base[name]
 
-        self.assertRaises(InvalidName, make_db, self.connection, "")
-        self.assertRaises(InvalidName, make_db, self.connection, "te$t")
-        self.assertRaises(InvalidName, make_db, self.connection, "te.t")
-        self.assertRaises(InvalidName, make_db, self.connection, "te\\t")
-        self.assertRaises(InvalidName, make_db, self.connection, "te/t")
-        self.assertRaises(InvalidName, make_db, self.connection, "te st")
+        self.assertRaises(InvalidName, make_db, self.client, "")
+        self.assertRaises(InvalidName, make_db, self.client, "te$t")
+        self.assertRaises(InvalidName, make_db, self.client, "te.t")
+        self.assertRaises(InvalidName, make_db, self.client, "te\\t")
+        self.assertRaises(InvalidName, make_db, self.client, "te/t")
+        self.assertRaises(InvalidName, make_db, self.client, "te st")
 
-        self.assertTrue(isinstance(self.connection.test, Database))
-        self.assertEqual(self.connection.test, self.connection["test"])
-        self.assertEqual(self.connection.test, Database(self.connection,
+        self.assertTrue(isinstance(self.client.test, Database))
+        self.assertEqual(self.client.test, self.client["test"])
+        self.assertEqual(self.client.test, Database(self.client,
                                                         "test"))
 
     def test_database_names(self):
-        self.connection.pymongo_test.test.save({"dummy": u"object"})
-        self.connection.pymongo_test_mike.test.save({"dummy": u"object"})
+        self.client.pymongo_test.test.save({"dummy": u"object"})
+        self.client.pymongo_test_mike.test.save({"dummy": u"object"})
 
-        dbs = self.connection.database_names()
+        dbs = self.client.database_names()
         self.assertTrue("pymongo_test" in dbs)
         self.assertTrue("pymongo_test_mike" in dbs)
 
     def test_drop_database(self):
-        self.assertRaises(TypeError, self.connection.drop_database, 5)
-        self.assertRaises(TypeError, self.connection.drop_database, None)
+        self.assertRaises(TypeError, self.client.drop_database, 5)
+        self.assertRaises(TypeError, self.client.drop_database, None)
 
         raise SkipTest("This test often fails due to SERVER-2329")
 
-        self.connection.pymongo_test.test.save({"dummy": u"object"}, safe=True)
-        dbs = self.connection.database_names()
+        self.client.pymongo_test.test.save({"dummy": u"object"})
+        dbs = self.client.database_names()
         self.assertTrue("pymongo_test" in dbs)
-        self.connection.drop_database("pymongo_test")
-        dbs = self.connection.database_names()
+        self.client.drop_database("pymongo_test")
+        dbs = self.client.database_names()
         self.assertTrue("pymongo_test" not in dbs)
 
-        self.connection.pymongo_test.test.save({"dummy": u"object"})
-        dbs = self.connection.database_names()
+        self.client.pymongo_test.test.save({"dummy": u"object"})
+        dbs = self.client.database_names()
         self.assertTrue("pymongo_test" in dbs)
-        self.connection.drop_database(self.connection.pymongo_test)
-        dbs = self.connection.database_names()
+        self.client.drop_database(self.client.pymongo_test)
+        dbs = self.client.database_names()
         self.assertTrue("pymongo_test" not in dbs)
 
     def test_iteration(self):
 
         def iterate():
-            [a for a in self.connection]
+            [a for a in self.client]
 
         self.assertRaises(TypeError, iterate)
 
     def test_insert_find_one_in_request(self):
         count = 0
         for i in range(100):
-            self.connection.start_request()
+            self.client.start_request()
             self.db.test.remove({})
             self.db.test.insert({"x": i})
             try:
@@ -246,44 +240,43 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
                     count += 1
             except:
                 count += 1
-            self.connection.end_request()
+            self.client.end_request()
         self.assertFalse(count)
 
     def test_nested_request(self):
-        conn = self.connection
+        client = self.client
 
         def assertRequest(in_request):
-            self.assertEqual(in_request, conn.in_request())
-            self.assertEqual(in_request, conn.master.in_request())
+            self.assertEqual(in_request, client.in_request())
+            self.assertEqual(in_request, client.master.in_request())
 
         # MasterSlaveConnection is special, alas - it has no auto_start_request
         # and it begins *not* in a request. When it's in a request, it sends
         # all queries to primary.
-        self.assertFalse(conn.in_request())
-        self.assertTrue(conn.master.in_request())
-        conn.master.end_request()
+        self.assertFalse(client.in_request())
+        self.assertFalse(client.master.in_request())
 
         # Start and end request
-        conn.start_request()
+        client.start_request()
         assertRequest(True)
-        conn.end_request()
+        client.end_request()
         assertRequest(False)
 
         # Double-nesting
-        conn.start_request()
-        conn.start_request()
-        conn.end_request()
+        client.start_request()
+        client.start_request()
+        client.end_request()
         assertRequest(True)
-        conn.end_request()
+        client.end_request()
         assertRequest(False)
 
     def test_request_threads(self):
-        conn = self.connection
+        client = self.client
 
         # In a request, all ops go through master
-        pool = conn.master._MongoClient__pool
-        conn.master.end_request()
-        self.assertNotInRequestAndDifferentSock(conn, pool)
+        pool = client.master._MongoClient__pool
+        client.master.end_request()
+        self.assertNotInRequestAndDifferentSock(client, pool)
 
         started_request, ended_request = threading.Event(), threading.Event()
         checked_request = threading.Event()
@@ -292,15 +285,15 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         # Starting a request in one thread doesn't put the other thread in a
         # request
         def f():
-            self.assertNotInRequestAndDifferentSock(conn, pool)
-            conn.start_request()
-            self.assertInRequestAndSameSock(conn, pool)
+            self.assertNotInRequestAndDifferentSock(client, pool)
+            client.start_request()
+            self.assertInRequestAndSameSock(client, pool)
             started_request.set()
             checked_request.wait()
             checked_request.clear()
-            self.assertInRequestAndSameSock(conn, pool)
-            conn.end_request()
-            self.assertNotInRequestAndDifferentSock(conn, pool)
+            self.assertInRequestAndSameSock(client, pool)
+            client.end_request()
+            self.assertNotInRequestAndDifferentSock(client, pool)
             ended_request.set()
             checked_request.wait()
             thread_done[0] = True
@@ -309,18 +302,18 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         t.setDaemon(True)
         t.start()
         started_request.wait()
-        self.assertNotInRequestAndDifferentSock(conn, pool)
+        self.assertNotInRequestAndDifferentSock(client, pool)
         checked_request.set()
         ended_request.wait()
-        self.assertNotInRequestAndDifferentSock(conn, pool)
+        self.assertNotInRequestAndDifferentSock(client, pool)
         checked_request.set()
         t.join()
-        self.assertNotInRequestAndDifferentSock(conn, pool)
+        self.assertNotInRequestAndDifferentSock(client, pool)
         self.assertTrue(thread_done[0], "Thread didn't complete")
 
     # This was failing because commands were being sent to the slaves
     def test_create_collection(self):
-        self.connection.pymongo_test.test.drop()
+        self.client.pymongo_test.test.drop()
 
         collection = self.db.create_collection('test')
         self.assertTrue(isinstance(collection, Collection))
@@ -329,12 +322,12 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
 
     # Believe this was failing for the same reason...
     def test_unique_index(self):
-        self.connection.pymongo_test.test.drop()
+        self.client.pymongo_test.test.drop()
         self.db.test.create_index('username', unique=True)
 
-        self.db.test.save({'username': 'mike'}, safe=True)
+        self.db.test.save({'username': 'mike'})
         self.assertRaises(OperationFailure,
-                          self.db.test.save, {'username': 'mike'}, safe=True)
+                          self.db.test.save, {'username': 'mike'})
 
     # NOTE this test is non-deterministic, but I expect
     # some failures unless the db is pulling instantaneously...
@@ -367,7 +360,7 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         self.assertFalse(count)
 
     def test_kill_cursor_explicit(self):
-        c = self.connection
+        c = self.client
         c.slave_okay = True
         db = c.pymongo_test
 
@@ -417,27 +410,26 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         self.assertRaises(OperationFailure, lambda: list(cursor2))
 
     def test_base_object(self):
-        c = self.connection
+        c = self.client
         self.assertFalse(c.slave_okay)
         self.assertTrue(bool(c.read_preference))
-        self.assertFalse(c.safe)
+        self.assertTrue(c.safe)
         self.assertEqual({}, c.get_lasterror_options())
         db = c.pymongo_test
         self.assertFalse(db.slave_okay)
         self.assertTrue(bool(c.read_preference))
-        self.assertFalse(db.safe)
+        self.assertTrue(db.safe)
         self.assertEqual({}, db.get_lasterror_options())
         coll = db.test
         coll.drop()
         self.assertFalse(coll.slave_okay)
         self.assertTrue(bool(c.read_preference))
-        self.assertFalse(coll.safe)
+        self.assertTrue(coll.safe)
         self.assertEqual({}, coll.get_lasterror_options())
         cursor = coll.find()
         self.assertFalse(cursor._Cursor__slave_okay)
         self.assertTrue(bool(cursor._Cursor__read_preference))
 
-        c.safe = True
         w = 1 + len(self.slaves)
         wtimeout=10000 # Wait 10 seconds for replication to complete
         c.set_lasterror_options(w=w, wtimeout=wtimeout)
@@ -466,19 +458,18 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
         coll.remove({'foo': 'bar'})
         self.assertEqual(0, coll.find({'foo': 'bar'}).count())
 
-        # Set self.connection back to defaults
         c.safe = False
         c.unset_lasterror_options()
-        self.assertFalse(self.connection.slave_okay)
-        self.assertTrue(bool(self.connection.read_preference))
-        self.assertFalse(self.connection.safe)
-        self.assertEqual({}, self.connection.get_lasterror_options())
+        self.assertFalse(self.client.slave_okay)
+        self.assertTrue(bool(self.client.read_preference))
+        self.assertFalse(self.client.safe)
+        self.assertEqual({}, self.client.get_lasterror_options())
 
     def test_document_class(self):
         c = MasterSlaveConnection(self.master, self.slaves)
         db = c.pymongo_test
         w = 1 + len(self.slaves)
-        db.test.insert({"x": 1}, safe=True, w=w)
+        db.test.insert({"x": 1}, w=w)
 
         self.assertEqual(dict, c.document_class)
         self.assertTrue(isinstance(db.test.find_one(), dict))
@@ -505,23 +496,23 @@ class TestMasterSlaveConnection(unittest.TestCase, TestRequestMixin):
 
     def test_tz_aware(self):
         dt = datetime.datetime.utcnow()
-        conn = MasterSlaveConnection(self.master, self.slaves)
-        self.assertEqual(False, conn.tz_aware)
-        db = conn.pymongo_test
+        client = MasterSlaveConnection(self.master, self.slaves)
+        self.assertEqual(False, client.tz_aware)
+        db = client.pymongo_test
         w = 1 + len(self.slaves)
-        db.tztest.insert({'dt': dt}, safe=True, w=w)
+        db.tztest.insert({'dt': dt}, w=w)
         self.assertEqual(None, db.tztest.find_one()['dt'].tzinfo)
 
-        conn = MasterSlaveConnection(self.master, self.slaves, tz_aware=True)
-        self.assertEqual(True, conn.tz_aware)
-        db = conn.pymongo_test
-        db.tztest.insert({'dt': dt}, safe=True, w=w)
+        client = MasterSlaveConnection(self.master, self.slaves, tz_aware=True)
+        self.assertEqual(True, client.tz_aware)
+        db = client.pymongo_test
+        db.tztest.insert({'dt': dt}, w=w)
         self.assertEqual(utc, db.tztest.find_one()['dt'].tzinfo)
 
-        conn = MasterSlaveConnection(self.master, self.slaves, tz_aware=False)
-        self.assertEqual(False, conn.tz_aware)
-        db = conn.pymongo_test
-        db.tztest.insert({'dt': dt}, safe=True, w=w)
+        client = MasterSlaveConnection(self.master, self.slaves, tz_aware=False)
+        self.assertEqual(False, client.tz_aware)
+        db = client.pymongo_test
+        db.tztest.insert({'dt': dt}, w=w)
         self.assertEqual(None, db.tztest.find_one()['dt'].tzinfo)
 
 
