@@ -30,7 +30,8 @@ from nose.plugins.skip import SkipTest
 import pymongo.pool
 from pymongo.mongo_client import MongoClient
 from pymongo.pool import Pool, NO_REQUEST, NO_SOCKET_YET, SocketInfo
-from pymongo.errors import ConfigurationError
+from pymongo.errors import ConfigurationError, ConnectionFailure
+from pymongo.errors import ExceededMaxWaiters
 from test import version, host, port
 from test.test_client import get_client
 from test.utils import delay, is_mongos, one
@@ -52,6 +53,18 @@ except ImportError:
     has_gevent = False
 
 
+def gc_collect_until_done(threads, timeout=60):
+    start = time.time()
+    running = list(threads)
+    while running:
+        assert (time.time() - start) < timeout, "Threads timed out"
+        for t in running:
+            t.thread.join(0.1)
+            if not t.alive:
+                running.remove(t)
+        gc.collect()
+
+
 class MongoThread(object):
     """A thread, or a greenlet, that uses a MongoClient"""
     def __init__(self, test_case):
@@ -67,18 +80,24 @@ class MongoThread(object):
             self.thread = Greenlet(self.run)
         else:
             self.thread = threading.Thread(target=self.run)
-            self.thread.setDaemon(True) # Don't hang whole test if thread hangs
-
+            self.thread.setDaemon(True)  # Don't hang whole test if thread hangs
 
         self.thread.start()
 
-    def join(self):
-        self.thread.join(300)
+    @property
+    def alive(self):
         if self.use_greenlets:
-            assert self.thread.dead, "Greenlet timeout"
+            return not self.thread.dead
         else:
-            assert not self.thread.isAlive(), "Thread timeout"
+            return self.thread.isAlive()
 
+    def join(self):
+        self.thread.join(10)
+        if self.use_greenlets:
+            msg = "Greenlet timeout"
+        else:
+            msg = "Thread timeout"
+        assert not self.alive, msg
         self.thread = None
 
     def run(self):
@@ -199,23 +218,32 @@ class OneOp(MongoThread):
 
 
 class CreateAndReleaseSocket(MongoThread):
+    """A thread or greenlet that acquires a socket, waits for all other threads
+    to reach rendezvous point, then terminates.
+    """
     class Rendezvous(object):
         def __init__(self, nthreads, use_greenlets):
             self.nthreads = nthreads
             self.nthreads_run = 0
+            self.use_greenlets = use_greenlets
             if use_greenlets:
                 self.lock = gevent.coros.RLock()
-                self.ready = gevent.event.Event()
             else:
                 self.lock = threading.Lock()
+            self.reset_ready()
+
+        def reset_ready(self):
+            if self.use_greenlets:
+                self.ready = gevent.event.Event()
+            else:
                 self.ready = threading.Event()
 
-    def __init__(self, ut, client, start_request, end_request, rendevous):
+    def __init__(self, ut, client, start_request, end_request, rendezvous):
         super(CreateAndReleaseSocket, self).__init__(ut)
         self.client = client
         self.start_request = start_request
         self.end_request = end_request
-        self.rendevous = rendevous
+        self.rendezvous = rendezvous
 
     def run_mongo_thread(self):
         # Do an operation that requires a socket.
@@ -229,7 +257,7 @@ class CreateAndReleaseSocket(MongoThread):
         self.client[DB].test.find_one()
 
         # Don't finish until all threads reach this point
-        r = self.rendevous
+        r = self.rendezvous
         r.lock.acquire()
         r.nthreads_run += 1
         if r.nthreads_run == r.nthreads:
@@ -238,13 +266,48 @@ class CreateAndReleaseSocket(MongoThread):
             r.lock.release()
         else:
             r.lock.release()
-            r.ready.wait(2) # Wait two seconds
+            r.ready.wait(2)  # Wait two seconds
             assert r.ready.isSet(), "Rendezvous timed out"
 
         for i in range(self.end_request):
             self.client.end_request()
 
 
+class CreateAndReleaseSocketNoRendezvous(MongoThread):
+    """A thread or greenlet that acquires a socket and terminates without
+    waiting for other threads to reach rendezvous point.
+    """
+    class Rendezvous(object):
+        def __init__(self, nthreads, use_greenlets):
+            self.nthreads = nthreads
+            self.nthreads_run = 0
+            if use_greenlets:
+                self.lock = gevent.coros.RLock()
+                self.ready = gevent.event.Event()
+            else:
+                self.lock = threading.Lock()
+                self.ready = threading.Event()
+
+    def __init__(self, ut, client, start_request, end_request):
+        super(CreateAndReleaseSocketNoRendezvous, self).__init__(ut)
+        self.client = client
+        self.start_request = start_request
+        self.end_request = end_request
+
+    def run_mongo_thread(self):
+        # Do an operation that requires a socket.
+        # test_max_pool_size uses this to spin up lots of threads requiring
+        # lots of simultaneous connections, to ensure that Pool obeys its
+        # max_size configuration and closes extra sockets as they're returned.
+        for i in range(self.start_request):
+            self.client.start_request()
+
+        # Use a socket
+        self.client[DB].test.find_one()
+        for i in range(self.end_request):
+            self.client.end_request()
+
+
 class _TestPoolingBase(object):
     """Base class for all client-pool tests. Doesn't inherit from
     unittest.TestCase, and its name is prefixed with "_" to avoid being
@@ -287,6 +350,12 @@ class _TestPoolingBase(object):
         kwargs['use_greenlets'] = self.use_greenlets
         return Pool(*args, **kwargs)
 
+    def sleep(self, seconds):
+        if self.use_greenlets:
+            gevent.sleep(seconds)
+        else:
+            time.sleep(seconds)
+
     def assert_no_request(self):
         self.assertEqual(
             NO_REQUEST, self.c._MongoClient__pool._get_request_state()
@@ -485,12 +554,10 @@ class _TestPooling(_TestPoolingBase):
     def test_pool_reuses_open_socket(self):
         # Test Pool's _check_closed() method doesn't close a healthy socket
         cx_pool = self.get_pool((host,port), 10, None, None, False)
+        cx_pool._check_interval_seconds = 0  # Always check.
         sock_info = cx_pool.get_socket()
         cx_pool.maybe_return_socket(sock_info)
 
-        # trigger _check_closed, which only runs on sockets that haven't been
-        # used in a second
-        time.sleep(1.1)
         new_sock_info = cx_pool.get_socket()
         self.assertEqual(sock_info, new_sock_info)
         cx_pool.maybe_return_socket(new_sock_info)
@@ -500,22 +567,23 @@ class _TestPooling(_TestPoolingBase):
         # Test that Pool removes dead socket and the socket doesn't return
         # itself PYTHON-344
         cx_pool = self.get_pool((host,port), 10, None, None, False)
+        cx_pool._check_interval_seconds = 0  # Always check.
         sock_info = cx_pool.get_socket()
 
         # Simulate a closed socket without telling the SocketInfo it's closed
         sock_info.sock.close()
         self.assertTrue(pymongo.pool._closed(sock_info.sock))
         cx_pool.maybe_return_socket(sock_info)
-        time.sleep(1.1) # trigger _check_closed
         new_sock_info = cx_pool.get_socket()
         self.assertEqual(0, len(cx_pool.sockets))
         self.assertNotEqual(sock_info, new_sock_info)
         cx_pool.maybe_return_socket(new_sock_info)
         self.assertEqual(1, len(cx_pool.sockets))
 
-    def test_pool_removes_dead_request_socket_after_1_sec(self):
+    def test_pool_removes_dead_request_socket_after_check(self):
         # Test that Pool keeps request going even if a socket dies in request
         cx_pool = self.get_pool((host,port), 10, None, None, False)
+        cx_pool._check_interval_seconds = 0  # Always check.
         cx_pool.start_request()
 
         # Get the request socket
@@ -524,7 +592,6 @@ class _TestPooling(_TestPoolingBase):
         self.assertEqual(sock_info, cx_pool._get_request_state())
         sock_info.sock.close()
         cx_pool.maybe_return_socket(sock_info)
-        time.sleep(1.1) # trigger _check_closed
 
         # Although the request socket died, we're still in a request with a
         # new socket
@@ -549,8 +616,8 @@ class _TestPooling(_TestPoolingBase):
         self.assertEqual(0, len(cx_pool.sockets))
         self.assertEqual(sock_info, cx_pool._get_request_state())
 
-        # Unlike in test_pool_removes_dead_request_socket_after_1_sec, we
-        # set sock_info.closed and *don't* wait 1 second
+        # Unlike in test_pool_removes_dead_request_socket_after_check, we
+        # set sock_info.closed and *don't* wait for it to be checked.
         sock_info.close()
         cx_pool.maybe_return_socket(sock_info)
 
@@ -571,11 +638,13 @@ class _TestPooling(_TestPoolingBase):
         # Test that Pool handles a socket dying that *used* to be the request
         # socket.
         cx_pool = self.get_pool((host,port), 10, None, None, False)
+        cx_pool._check_interval_seconds = 0  # Always check.
         cx_pool.start_request()
 
         # Get the request socket
         sock_info = cx_pool.get_socket()
         self.assertEqual(sock_info, cx_pool._get_request_state())
+        cx_pool.maybe_return_socket(sock_info)
 
         # End request
         cx_pool.end_request()
@@ -583,8 +652,6 @@ class _TestPooling(_TestPoolingBase):
 
         # Kill old request socket
         sock_info.sock.close()
-        cx_pool.maybe_return_socket(sock_info)
-        time.sleep(1.1) # trigger _check_closed
 
         # Dead socket detected and removed
         new_sock_info = cx_pool.get_socket()
@@ -595,6 +662,32 @@ class _TestPooling(_TestPoolingBase):
         cx_pool.maybe_return_socket(new_sock_info)
         self.assertEqual(1, len(cx_pool.sockets))
 
+    def test_dead_request_socket_with_max_size(self):
+        # When a pool replaces a dead request socket, the semaphore it uses
+        # to enforce max_size should remain unaffected.
+        cx_pool = self.get_pool(
+            (host, port), 1, None, None, False, wait_queue_timeout=1)
+
+        cx_pool._check_interval_seconds = 0  # Always check.
+        cx_pool.start_request()
+
+        # Get and close the request socket.
+        request_sock_info = cx_pool.get_socket()
+        request_sock_info.sock.close()
+        cx_pool.maybe_return_socket(request_sock_info)
+
+        # Detects closed socket and creates new one, semaphore value still 0.
+        request_sock_info_2 = cx_pool.get_socket()
+        self.assertNotEqual(request_sock_info, request_sock_info_2)
+        cx_pool.maybe_return_socket(request_sock_info_2)
+        cx_pool.end_request()
+
+        # Semaphore value now 1; we can get a socket.
+        sock_info = cx_pool.get_socket()
+
+        # Clean up.
+        cx_pool.maybe_return_socket(sock_info)
+
     def test_socket_reclamation(self):
         if sys.platform.startswith('java'):
             raise SkipTest("Jython can't do socket reclamation")
@@ -631,6 +724,10 @@ class _TestPooling(_TestPoolingBase):
             g.start()
             g.join(1)
             self.assertTrue(g.ready(), "Greenlet is hung")
+
+            # In Gevent after 0.13.8, join() returns before the Greenlet.link
+            # callback fires. Give it a moment to reclaim the socket.
+            gevent.sleep(0.1)
         else:
             lock = thread.allocate_lock()
             lock.acquire()
@@ -652,10 +749,10 @@ class _TestPooling(_TestPoolingBase):
             # Access the thread local from the main thread to trigger the
             # ThreadVigil's delete callback, returning the request socket to
             # the pool.
-            # In Python 2.6 and lesser, a dead thread's locals are deleted
+            # In Python 2.7.0 and lesser, a dead thread's locals are deleted
             # and those locals' weakref callbacks are fired only when another
-            # thread accesses the locals and finds the thread state is stale.
-            # This is more or less a bug in Python <= 2.6. Accessing the thread
+            # thread accesses the locals and finds the thread state is stale,
+            # see http://bugs.python.org/issue1868. Accessing the thread
             # local from the main thread is a necessary part of this test, and
             # realistic: in a multithreaded web server a new thread will access
             # Pool._ident._local soon after an old thread has died.
@@ -664,6 +761,7 @@ class _TestPooling(_TestPoolingBase):
         # Pool reclaimed the socket
         self.assertEqual(1, len(cx_pool.sockets))
         self.assertEqual(the_sock[0], id(one(cx_pool.sockets).sock))
+        self.assertEqual(0, len(cx_pool._tid_to_sock))
 
 
 class _TestMaxPoolSize(_TestPoolingBase):
@@ -671,28 +769,52 @@ class _TestMaxPoolSize(_TestPoolingBase):
     no matter how start/end_request are called. To be run both with threads and
     with greenlets.
     """
-    def _test_max_pool_size(self, start_request, end_request):
-        c = self.get_client(max_pool_size=4, auto_start_request=False)
-        # If you increase nthreads over about 35, note a
-        # Gevent 0.13.6 bug on Mac, Greenlet.join() hangs if more than
-        # about 35 Greenlets share a MongoClient. Apparently fixed in
-        # recent Gevent development.
-        nthreads = 10
+    def _test_max_pool_size(
+            self, start_request, end_request, max_pool_size=4, nthreads=10):
+        """Start `nthreads` threads. Each calls start_request `start_request`
+        times, then find_one and waits at a barrier; once all reach the barrier
+        each calls end_request `end_request` times. The test asserts that the
+        pool ends with min(max_pool_size, nthreads) sockets or, if
+        start_request wasn't called, at least one socket.
 
-        rendevous = CreateAndReleaseSocket.Rendezvous(
+        This tests both max_pool_size enforcement and that leaked request
+        sockets are eventually returned to the pool when their threads end.
+
+        You may need to increase ulimit -n on Mac.
+
+        If you increase nthreads over about 35, note a
+        Gevent 0.13.6 bug on Mac: Greenlet.join() hangs if more than
+        about 35 Greenlets share a MongoClient. Apparently fixed in
+        recent Gevent development.
+        """
+        if start_request:
+            assert max_pool_size >= nthreads, "Deadlock"
+
+        c = self.get_client(
+            max_pool_size=max_pool_size, auto_start_request=False)
+
+        rendezvous = CreateAndReleaseSocket.Rendezvous(
             nthreads, self.use_greenlets)
 
         threads = []
         for i in range(nthreads):
             t = CreateAndReleaseSocket(
-                self, c, start_request, end_request, rendevous)
+                self, c, start_request, end_request, rendezvous)
             threads.append(t)
 
         for t in threads:
             t.start()
 
-        for t in threads:
-            t.join()
+        if 'PyPy' in sys.version:
+            # With PyPy we need to kick off the gc whenever the threads hit the
+            # rendezvous since nthreads > max_pool_size.
+            gc_collect_until_done(threads)
+        else:
+            for t in threads:
+                t.join()
+
+        # join() returns before the thread state is cleared; give it time.
+        self.sleep(1)
 
         for t in threads:
             self.assertTrue(t.passed)
@@ -716,34 +838,272 @@ class _TestMaxPoolSize(_TestPoolingBase):
                     the_hub.shutdown()
 
             if start_request:
-                self.assertEqual(4, len(cx_pool.sockets))
+                # Trigger final cleanup in Python <= 2.7.0.
+                cx_pool._ident.get()
+                expected_idle = min(max_pool_size, nthreads)
+                message = (
+                    '%d idle sockets (expected %d) and %d request sockets'
+                    ' (expected 0)' % (
+                        len(cx_pool.sockets), expected_idle,
+                        len(cx_pool._tid_to_sock)))
+
+                self.assertEqual(
+                    expected_idle, len(cx_pool.sockets), message)
             else:
                 # Without calling start_request(), threads can safely share
-                # sockets; the number running concurrently, and hence the number
-                # of sockets needed, is between 1 and 10, depending on thread-
-                # scheduling.
+                # sockets; the number running concurrently, and hence the
+                # number of sockets needed, is between 1 and 10, depending
+                # on thread-scheduling.
                 self.assertTrue(len(cx_pool.sockets) >= 1)
 
+            # thread.join completes slightly *before* thread locals are
+            # cleaned up, so wait up to 5 seconds for them.
+            self.sleep(0.1)
+            cx_pool._ident.get()
+            start = time.time()
+
+            while (
+                not cx_pool.sockets
+                and cx_pool._socket_semaphore.counter < max_pool_size
+                and (time.time() - start) < 5
+            ):
+                self.sleep(0.1)
+                cx_pool._ident.get()
+
+            self.assertEqual(max_pool_size, cx_pool._socket_semaphore.counter)
+            self.assertEqual(0, len(cx_pool._tid_to_sock))
+
+    def _test_max_pool_size_no_rendezvous(self, start_request, end_request):
+        max_pool_size = 5
+        c = self.get_client(
+            max_pool_size=max_pool_size, auto_start_request=False)
+
+        # If you increase nthreads over about 35, note a
+        # Gevent 0.13.6 bug on Mac, Greenlet.join() hangs if more than
+        # about 35 Greenlets share a MongoClient. Apparently fixed in
+        # recent Gevent development.
+
+        # On the other hand, nthreads had better be much larger than
+        # max_pool_size to ensure that max_pool_size sockets are actually
+        # required at some point in this test's execution.
+        nthreads = 30
+
+        if (sys.platform.startswith('java')
+                and start_request > end_request
+                and nthreads > max_pool_size):
+
+            # Since Jython can't reclaim the socket and release the semaphore
+            # after a thread leaks a request, we'll exhaust the semaphore and
+            # deadlock.
+            raise SkipTest("Jython can't do socket reclamation")
+
+        threads = []
+        for i in range(nthreads):
+            t = CreateAndReleaseSocketNoRendezvous(
+                self, c, start_request, end_request)
+            threads.append(t)
+
+        for t in threads:
+            t.start()
+
+        if 'PyPy' in sys.version:
+            # With PyPy we need to kick off the gc whenever the threads hit the
+            # rendezvous since nthreads > max_pool_size.
+            gc_collect_until_done(threads)
+        else:
+            for t in threads:
+                t.join()
+
+        for t in threads:
+            self.assertTrue(t.passed)
+
+        cx_pool = c._MongoClient__pool
+
+        # Socket-reclamation depends on timely garbage-collection
+        if 'PyPy' in sys.version:
+            gc.collect()
+
+        if self.use_greenlets:
+            # Wait for Greenlet.link() callbacks to execute
+            the_hub = hub.get_hub()
+            if hasattr(the_hub, 'join'):
+                # Gevent 1.0
+                the_hub.join()
+            else:
+                # Gevent 0.13 and less
+                the_hub.shutdown()
+
+        # thread.join completes slightly *before* thread locals are
+        # cleaned up, so wait up to 5 seconds for them.
+        self.sleep(0.1)
+        cx_pool._ident.get()
+        start = time.time()
+
+        while (
+            not cx_pool.sockets
+            and cx_pool._socket_semaphore.counter < max_pool_size
+            and (time.time() - start) < 5
+        ):
+            self.sleep(0.1)
+            cx_pool._ident.get()
+
+        self.assertTrue(len(cx_pool.sockets) >= 1)
+        self.assertEqual(max_pool_size, cx_pool._socket_semaphore.counter)
+
     def test_max_pool_size(self):
-        self._test_max_pool_size(0, 0)
+        self._test_max_pool_size(
+            start_request=0, end_request=0, nthreads=10, max_pool_size=4)
 
     def test_max_pool_size_with_request(self):
-        self._test_max_pool_size(1, 1)
+        self._test_max_pool_size(
+            start_request=1, end_request=1, nthreads=10, max_pool_size=10)
+
+    def test_max_pool_size_with_multiple_request(self):
+        self._test_max_pool_size(
+            start_request=10, end_request=10, nthreads=10, max_pool_size=10)
 
     def test_max_pool_size_with_redundant_request(self):
-        self._test_max_pool_size(2, 1)
-        self._test_max_pool_size(20, 1)
+        self._test_max_pool_size(
+            start_request=2, end_request=1, nthreads=10, max_pool_size=10)
+
+    def test_max_pool_size_with_redundant_request2(self):
+        self._test_max_pool_size(
+            start_request=20, end_request=1, nthreads=10, max_pool_size=10)
+
+    def test_max_pool_size_with_redundant_request_no_rendezvous(self):
+        self._test_max_pool_size_no_rendezvous(2, 1)
+
+    def test_max_pool_size_with_redundant_request_no_rendezvous2(self):
+        self._test_max_pool_size_no_rendezvous(20, 1)
 
     def test_max_pool_size_with_leaked_request(self):
         # Call start_request() but not end_request() -- when threads die, they
         # should return their request sockets to the pool.
-        self._test_max_pool_size(1, 0)
+        self._test_max_pool_size(
+            start_request=1, end_request=0, nthreads=10, max_pool_size=10)
+
+    def test_max_pool_size_with_leaked_request_no_rendezvous(self):
+        self._test_max_pool_size_no_rendezvous(1, 0)
 
     def test_max_pool_size_with_end_request_only(self):
         # Call end_request() but not start_request()
         self._test_max_pool_size(0, 1)
 
 
+class SocketGetter(MongoThread):
+    """Utility for _TestMaxOpenSockets and _TestWaitQueueMultiple"""
+    def __init__(self, test_case, pool):
+        super(SocketGetter, self).__init__(test_case)
+        self.state = 'init'
+        self.pool = pool
+        self.sock = None
+
+    def run(self):
+        self.state = 'get_socket'
+        self.sock = self.pool.get_socket()
+        self.state = 'sock'
+
+
+class _TestMaxOpenSockets(_TestPoolingBase):
+    """Test that connection pool doesn't open more than max_size sockets.
+    To be run both with threads and with greenlets.
+    """
+    def get_pool_with_wait_queue_timeout(self, wait_queue_timeout):
+        return self.get_pool(('127.0.0.1', 27017),
+                             1, None, None,
+                             False,
+                             wait_queue_timeout=wait_queue_timeout,
+                             wait_queue_multiple=None)
+
+    def test_wait_queue_timeout(self):
+        wait_queue_timeout = 2  # Seconds
+        pool = self.get_pool_with_wait_queue_timeout(wait_queue_timeout)
+        sock_info = pool.get_socket()
+        start = time.time()
+        self.assertRaises(ConnectionFailure, pool.get_socket)
+        duration = time.time() - start
+        self.assertTrue(
+            abs(wait_queue_timeout - duration) < 1,
+            "Waited %.2f seconds for a socket, expected %f" % (
+                duration, wait_queue_timeout))
+
+        sock_info.close()
+
+    def test_blocking(self):
+        # Verify get_socket() with no wait_queue_timeout blocks forever.
+        pool = self.get_pool_with_wait_queue_timeout(None)
+
+        # Reach max_size.
+        s1 = pool.get_socket()
+        t = SocketGetter(self, pool)
+        t.start()
+        while t.state != 'get_socket':
+            self.sleep(0.1)
+
+        self.sleep(1)
+        self.assertEqual(t.state, 'get_socket')
+        pool.maybe_return_socket(s1)
+        while t.state != 'sock':
+            self.sleep(0.1)
+
+        self.assertEqual(t.state, 'sock')
+        self.assertEqual(t.sock, s1)
+        s1.close()
+
+
+class _TestWaitQueueMultiple(_TestPoolingBase):
+    """Test that connection pool doesn't allow more than
+    waitQueueMultiple * max_size waiters.
+    To be run both with threads and with greenlets.
+    """
+    def get_pool_with_wait_queue_multiple(self, wait_queue_multiple):
+        return self.get_pool(('127.0.0.1', 27017),
+                             2, None, None,
+                             False,
+                             wait_queue_timeout=None,
+                             wait_queue_multiple=wait_queue_multiple)
+
+    def test_wait_queue_multiple(self):
+        pool = self.get_pool_with_wait_queue_multiple(3)
+
+        # Reach max_size sockets.
+        socket_info_0 = pool.get_socket()
+        socket_info_1 = pool.get_socket()
+
+        # Reach max_size * wait_queue_multiple waiters.
+        threads = []
+        for _ in xrange(6):
+            t = SocketGetter(self, pool)
+            t.start()
+            threads.append(t)
+
+        self.sleep(1)
+        for t in threads:
+            self.assertEqual(t.state, 'get_socket')
+
+        self.assertRaises(ExceededMaxWaiters, pool.get_socket)
+        socket_info_0.close()
+        socket_info_1.close()
+
+    def test_wait_queue_multiple_unset(self):
+        pool = self.get_pool_with_wait_queue_multiple(None)
+        socks = []
+        for _ in xrange(2):
+            sock = pool.get_socket()
+            socks.append(sock)
+        threads = []
+        for _ in xrange(30):
+            t = SocketGetter(self, pool)
+            t.start()
+            threads.append(t)
+        self.sleep(1)
+        for t in threads:
+            self.assertEqual(t.state, 'get_socket')
+
+        for socket_info in socks:
+            socket_info.close()
+
+
 class _TestPoolSocketSharing(_TestPoolingBase):
     """Directly test that two simultaneous operations don't share a socket. To
     be run both with threads and with greenlets.
